<?php
/**
 * Database interaction class.
 * 
 * This class is responsible for interacting with the database.  
 * Currently only 'mysql' and 'mysqli' extensions are supported.
 * 
 * @package Database
 * @category Databases
 * @author Misa Brezanac <brezanac@gmail.com>
 * @copyright http://creativecommons.org/licenses/by-nc-sa/3.0/rs/
*/
class Database {
    /**
     * Database connection handle
    */
    protected $mysqldb;
    
    /**
     * The query result containing variable
    */
    protected $queryResult;
    
    /**
     * Query initiation timestamp     
    */
    protected $queryInitiationTimestamp;
    
    /**
     * Query completion timestamp
    */
    protected $queryCompletionTimestamp;
    
    /**
     * Calculated query execution time
    */
    protected $queryExecutionTime;
    
    /**
     * Number of executed queries
    */
    protected $queryNumberOfExecutedQueries;
    
    /**
     * Number of affected rows
    */
    protected $queryNumberOfAffectedRows;
    
    /**
     * Database class constructor.
     * 
     * Establishes a connection to the database and returns a Database connection handle.
     * 
    */
    public function __construct(){
        try {
            /*
             * Depending on the type of the selected connection type a different connection approach is used.
            */
            switch(DB_TYPE){
                case 'mysqli':
                    /*
                     * Establishing connection to the database. 
                    */
                    if($this->mysqldb = new mysqli(DB_HOST, DB_USERNAME, DB_PASSWORD)){  
                        /*
                         * Selecting the database
                        */
                        if(!$this->mysqldb->select_db(DB_NAME)){
                            throw new Custom_Exception('db_mysqli_unable_to_select_db');
                        }
                    } else {
                        throw new Custom_Exception('db_mysqli_unable_to_connect');       
                    }     
                break;
                default:
                    /*
                     * Establishing connection to the database. 
                    */
                    if($this->mysqldb = @mysql_connect(DB_HOST, DB_USERNAME, DB_PASSWORD)){
                        /*
                         * Selecting the database
                        */
                        if(!(@mysql_select_db(DB_NAME, $this->mysqldb)))
                            throw new Custom_Exception('db_mysql_unable_to_select_db');            
                    } else {
                         throw new Custom_Exception('db_mysql_unable_to_connect');       
                    }                     
            }  
        } catch (Custom_Exception $e){
            /*
             * An error was triggered
            */  
            echo $e->noteMessage();  
        } 
    } 
    
    /**
     * Executes an SQL query
     * 
     * @param string @sql The SQL query to be executed
     * @return resource Either a resource link of the query result or false.
    */
    public function executeQuery($sql){
        $this->queryInitiationTimestamp = $this->getCurrentTimestamp(); //noting the query initiation timestamp  
        ++$this->queryNumberOfExecutedQueries; //incrementing the number of executed queries
        try {
            switch(DB_TYPE){
                case 'mysqli':
                    $this->mysqldb->set_charset(DB_CHARSET); //setting the database connection charset prior to a query
                    if(!($this->queryResult = $this->mysqldb->query($sql))){
                        throw new Custom_Exception('db_mysqli_query_unsuccessful');           
                    }
                break;
                default:
                    @mysql_set_charset(DB_CHARSET, $this->mysqldb); //setting the connection charset
                    if(!($this->queryResult = @mysql_query($sql, $this->mysqldb))){
                        throw new Custom_Exception('db_mysql_query_unsuccessful'); 
                    } 
            }   
        } catch (Custom_Exception $e){
            $e->noteMessage();      
        }
        $this->queryCompletionTimestamp = $this->getCurrentTimestamp(); //setting the completion timestamp
        $this->queryExecutionTime = $this->queryCompletionTimestamp - $this->queryInitiationTimestamp; //calculating the execution time
        return $this->queryResult; //returning the result set
    }
    
    /**
     * Returns the current timestamp on the server with the requested microseconds precission.
     * 
     * @return float The current timestamp with a requested microseconds precission.
    */
    public function getCurrentTimestamp(){
        list($microseconds, $seconds) = explode(" ", microtime());        
        return((double)$seconds + (double)$microseconds);
    }
    
    /**
     * Returns the number of affected rows for the last query
     * 
     * @return integer The number of affected rows for the last query.
    */
    public function getAffectedRows(){
        switch(DB_TYPE){
            case 'mysqli':
                $this->queryNumberOfAffectedRows = @mysqli_affected_rows($this->mysqldb);   
            break;
            default:
                $this->queryNumberOfAffectedRows = @mysql_affected_rows($this->mysqldb);    
        }    
    }
    
    /**
     * Returns a specific type of data inside an array according to the specified required type.
     * 
     * Allowed types are 'object', 'array' and 'assoc'. Default is 'assoc'.
     * 
     * @param string $type The requested type of returned data
     * @return array The result set.
    */
    public function getFormattedResults($type = 'assoc'){
        $this->resultArray = array();
        $allowedTypes = array('array', 'object', 'assoc'); //list of allowed 'modes'
        try {
            if(in_array($type, $allowedTypes)){
                switch(DB_TYPE){
                    case 'mysqli': //mysqli type of connection
                    $tempFunctionString = "mysqli_fetch_".$type; //used only to pass as a dinamically generated function  
                    /*
                    * WARNING!!! Dynamically generated function name
                    */
                    while($currentRow = $tempFunctionString($this->queryResult)){ 
                        $this->resultArray[] = $currentRow;        
                    }
                    break;
                    default:
                        $tempFunctionString = "mysql_fetch_".$type; //used only to pass as a dinamically generated function  
                        /*
                        * WARNING!!! Dynamically generated function name
                        */
                        while($currentRow = $tempFunctionString($this->queryResult)){
                            $this->resultArray[] = $currentRow;        
                        }             
                }
            } else {
                throw new Custom_Exception('db_get_fomatted_results_invalid_type');
            }   
        } catch (Custom_Exception $e){
            $e->noteMessage(); //'noting' the message in the Exception handler      
        }
        return $this->resultArray;
    }
    
    /**
     * Counts the number of fetched records in a MySQL result set. 
     * 
     * @return integer The number of fetched records.
    */
    public function getNumRows(){
        switch(DB_TYPE){
            case 'mysqli':
                return @mysqli_num_rows($this->queryResult);
            break;
            default:
                return @mysql_num_rows($this->queryResult);        
        }               
    }
}
?>
