<?php
/**
 * This is the Eastern Orthodox Church calendar class that extends the base Calendar class. 
 * 
 * The purpose of this class is to allow an easy creation of the Eastern Orthodox Church calendars. The class itself
 * is capable of performing all the required calculations along with the required types of responses 
 * (AJAX and non-AJAX based).
 * NOTICE: This class requires to be invoked as an extension class to the class 'Calendar'. Therefore it is dependent
 * upon the following classes:
 * 'Calendar' - used for some general Calendar calculations
 * 'Custom_Exceptions' - used to extend the native 'Exceptions' PHP class
 * 'Database' - required for database interaction
 * 
 * @package Calendar
 * @subpackage Orthodox calendar
 * @category Calendars
 * @author Misa Brezanac <brezanac@gmail.com>
 * @copyright http://creativecommons.org/licenses/by-nc-sa/3.0/rs/
*/
class Orthodox_Calendar extends Calendar { 
    /**
     * Database table name for the 'fixed feasts' data
    */
    const DB_TABLE_FIXED_FEASTS = "fixedfeasts";
    
    /**
     * Database table name for the 'moveable feasts' data
    */
    const DB_TABLE_MOVEABLE_FEASTS = "moveablefeasts";
    
    /**
     * Database connection handle
    */
    private $mysqldb;
    
    /**
     * Database query results resource link
    */
    private $result;
     
    /**
     * An associative array containing the 'fixed feasts' for the current month.
    */
    private $loadedFixedFeasts;
     
    /**
     * An assocciative array containing the 'moveable feasts' for the entire year.
    */
    private $loadedMoveableFeasts;
     
    /**
     * An associative array containing the calculated 'moveable feasts' for the current year. 
    */
    private $moveableFeasts;
     
    /**
     * A final list of feasts compiled of fixed and calculated moveable feasts for the current month
    */
    private $finalFeastsList;
    
    /**
     * Easter timestamp representing Easter day noon.
    */
    private $easterTimestamp;
     
    /**
     * The numerical representation of the Easter day position in month.
    */
    private $easterDay;
     
    /**                                                    
     * The numerical representation of the Easter day month of the year. 
    */
    private $easterMonth;
     
    /**
     * Serbian weekdays abbreviations (long description, short description) 
    */
    private static $days = array(
        array('недеља', 'НЕ'),
        array('понедељак', 'ПО'),
        array('уторак', 'У'),
        array('среда', 'СР'),
        array('четвртак', 'Ч'),
        array('петак', 'ПЕ'),
        array('субота', 'СУ')
     );

    /**
     * Serbian month names (long description, short description) 
    */
    private static $months = array(
        array('Jануар', 'Јan'),
        array('Фебруар', 'Феб'),
        array('Mарт', 'Мар'),
        array('Aприл', 'Апр'),
        array('Mај', 'Мај'),
        array('Jун', 'Јун'),
        array('Јул', 'Јул'),
        array('Август', 'Авг'),
        array('Септембар', 'Сеп'),
        array('Октобар', 'Окт'),
        array('Новембар', 'Нов'),
        array('Децембар', 'Дец')
    );
    
    /**
     * Serbian singular and plural abbreviations for the word 'day'
    */
    private static $abbrDays = array('дан', 'дана');
     
    /**
     * Used for temporary CSS class assignment of the table rows during calendar rendering 
    */
    private $rowClass;
     
    /**
     * Used for temporary class assignment of the table content cell during calendar rendering
    */ 
    private $contentClass;
    
    /**
     * Used for temporary assignment of previous 'title' content during the process of feasts title merging
    */
    private $titleTemp;
     
    /**
     * Current 'travel into the past' day value
    */
    private $tempOffsetDay; 
    
    /**
     * Current 'travel into the past' month value
    */
    private $tempOffsetMonth;
    
    /**
     * Current 'travel into the past' year value
    */
    private $tempOffsetYear;
    
    /**
    * How many days into the past are we traveling in the current loop?
    */
    private $offsetCounter;
    
    /**
     * The number of successful 'weekday' hits during the 'travel into the past'
    */
    private $hitCount;
    
    /**
     * A temporary counter variable
    */
    private $tempCounter;
    
    /**
     * A temporary storage array
    */
    private $tempLoadedData;
    
    /**
     * An array containig a detailed description and all the info of the feast intended for the infoPanel
    */
    private $details;
    
    /**
     * The number of feasts stored in the database
    */
    private $databaseFeastsCount;
    
    /**
     * The class fields retrieval accessor (getter)
     * 
     * In case a certain variable declared with a limited scope is supposed to be accessed we use this 'getter'
     * method to retrieve it. 
     * 
     * @param string $valueName The name of the class field which value is to be retrieved
     * @return string The requested value
    */
    public function getValue($valueName){
        return $this->$valueName ;
    }
    
    /**
    *  The actual Eastern Orthodox church calendar building function. 
    * 
    * First it calls the 'loadDays()' method to load the data from the database. 
    * After that the 'calculateMoveableFeasts()' is called to calculate the 'moveable' feasts that will replace
    * the corresponding 'fixed' day feasts. At the end the required replacements and merges are made. 
    */
    public function buildCalendar(){
        parent::buildCalendar(); //just requesting some goodies from the 'parent' class
        $this->loadDays();
        $this->calculateEaster();
        $this->calculateMoveableFeasts();
        $this->mergeFeasts();        
    }
    
    /**
    *  Calculates the 'Julian day' according to the old 'Jualian' calendar.
    * 
    * @param integer $day The day of the current month for which the calculations are made.
    * @return integer Converted 'Julian' day
    */
    private function calculateJulian($day){
        return date("j", mktime(12, 0, 0, $this->month, $day, $this->year) - 1123200);   
    }
    
    /**
    *  Calculates the exact date, month and timestamp of the Orthodox Easter for the current year. 
    * The timestamp is calculated for the Easter day noon to avoid certain problems with time zones.
    */
    public function calculateEaster(){
        $this->easterDay = (15 + (int)($this->year % 19) * 19) % 30; //calculating some strange stuff
        $this->easterDay += 10 - ($this->easterDay + $this->year + $this->year / 4) % 7; //even more 'some strange stuff' 
        
        if($this->easterDay > 30){
            $this->easterMonth = 5; //the Easter is in May
            $this->easterDay -= 30;   
        } else {
            $this->easterMonth = 4; //the Easter is in April   
        } 
    
        $this->easterTimestamp = mktime(12, 0, 0, $this->easterMonth, $this->easterDay, $this->year);
    }
    
    /**
    *  Loads external 'fixed' and 'moveable' feasts data from the database.
    * 
    * This function retrieves the 'fixed' and 'moveable' days data from the external data storages. 
    * NOTICE: Currently only database storages are supported. 
    */
    public function loadDays(){
        /*
        * First we retrieve the 'fixed' days data. That's a little bit trickier since we need to take care of 
        * the possibility that the current year might be a leap year.
        */
        $this->mysqldb = new Database();
        
        if($this->isLeapYear()){
            /*
            * In case of a leap year we dont't need to do anything except to load the data directly from the database
            * since the database is initialy constructed to provide direct leap year data. 
            */
            $sql = "SELECT id, day, month, title, isRedLetter, important FROM ".self::DB_TABLE_FIXED_FEASTS." WHERE month=".$this->month;
        } else {
            switch($this->month){
                case 2: 
                    /*
                    * A non-leap year February
                    */
                    $sql = "SELECT id, day, month, title, isRedLetter, important FROM ".self::DB_TABLE_FIXED_FEASTS." WHERE month=".$this->month." LIMIT 28";           
                break;
                case 3:
                    /*
                    * A non-leap year March. First we load the 29th february and then the first 13 days of March
                    */
                    $sql = "SELECT id, title, 3 AS month, isRedLetter, important FROM `fixedfeasts` WHERE id BETWEEN 60 AND 72";
                    $this->result = $this->mysqldb->executeQuery($sql);
                    $this->tempLoadedData = $this->mysqldb->getFormattedResults();
                    
                    /*
                    * Now we store the temporary data into the end result array and go for another run to load the rest
                    * of the March days without the 14th of march 'feast' which appears only on leap years.
                    * Before that we need to mark every row with an autoincrementing 'day' number.
                    */
                    $this->tempCounter = 1; //number assignement beggins from 1
                    foreach($this->tempLoadedData as $key => $value){
                        $this->tempLoadedData[$key]['day'] = $this->tempCounter++;
                    }
                    
                    $this->loadedFixedFeasts = $this->tempLoadedData;
                    
                    /*
                    * Loading the rest of the days.
                    */
                    $sql = "SELECT id, title, 3 AS month, isRedLetter, important FROM `fixedfeasts` WHERE id BETWEEN 74 AND 91";
                    $this->result = $this->mysqldb->executeQuery($sql);
                    $this->tempLoadedData = $this->mysqldb->getFormattedResults();
                    
                    /*
                    * Row marking and merging of the second part with the first one.
                    */
                    $this->tempCounter = 14; //number assignment beggins from number 14
                    foreach($this->tempLoadedData as $key => $value){
                        $this->tempLoadedData[$key]['day'] = $this->tempCounter++;
                        $this->loadedFixedFeasts[$this->tempCounter] = $this->tempLoadedData[$key]; 
                    }  
                break;
                default;
                    /*
                    *  The following SQL applies to all the other non leap year cases
                    */
                    $sql = "SELECT id, day, month, title, isRedLetter, important FROM ".self::DB_TABLE_FIXED_FEASTS." WHERE month=".$this->month;
            }       
        }
        
        /*
        * Performing the actual query if no results were already retrieved in the non-leap March block 
        */
        if(!count($this->loadedFixedFeasts)){
            $this->result = $this->mysqldb->executeQuery($sql);
            $this->loadedFixedFeasts = $this->mysqldb->getFormattedResults();
        } 

         /*
         * Then we retrieve the 'moveable days' data and finalize the actual 'moveable' feasts array
         */
         $sql = "SELECT id, title, bindingDate, bindOffset, isRedLetter, isSubstitution FROM ".self::DB_TABLE_MOVEABLE_FEASTS;
         $this->result = $this->mysqldb->executeQuery($sql); 
         $this->loadedMoveableFeasts = $this->mysqldb->getFormattedResults();
    }
    
    /**
     * Retrieves the number of feasts in both tables from the database ('fixed' and 'moveable')
     * 
     * @return integer The number of records in both database tables.
    */
    public function getNumberOfFeasts(){
        $this->mysqldb = new Database(); 
        
        /* 
         * The number of records (days) from the 'fixed' days table
        */
        $sql = "SELECT id FROM ".self::DB_TABLE_FIXED_FEASTS;
        $this->result = $this->mysqldb->executeQuery($sql);
        $this->databaseFeastsCount = $this->mysqldb->getNumRows();
        
        /*
         * The number of records (days) from the 'moveable' days table    
        */
        $sql = "SELECT id FROM ".self::DB_TABLE_MOVEABLE_FEASTS;
        $this->result = $this->mysqldb->executeQuery($sql);
        
        /*
         * Now we can add the number of 'moveable' feasts directlly to the number of the 'fixed' ones
        */
        $this->databaseFeastsCount += $this->mysqldb->getNumRows();
    }
    
    /**
    *  Calculates the 'moveable feasts' for the current year.
    * 
    * This method calculates the 'moveable' feasts that depend upon specific calculation algorithms.
    * The calculated 'moveable' feasts will replace the corresponding feasts in the 'fixed' feasts list.
    */
    public function calculateMoveableFeasts(){
        
        /*
        * First the exact day and month of the Easter is calculated since almost all of the 'moveable' feasts depend
        * upon it.
        */
        $this->calculateEaster();
        
        foreach($this->loadedMoveableFeasts as $feast => $values){
            if(!intval($values['bindingDate'])){
                /*
                * Here we are calculating 'moveable' feasts positions by using the Easter day as a marker and data
                * about each feast's time offset from the Easter day. Time offset for every feast is defined in the 
                * database.
                * These kind of Eastern dependant feasts have a value of '0' for 'bindingDate' column accompanied 
                * with an appropriate 'bindOffset' value which is used to calculate the exact position of it for the 
                * current year. 
                */
                $this->tempOffsetTimestamp = $this->easterTimestamp + $values['bindOffset'] * 86400; 
                list($this->newDay, $this->newMonth) = explode(',',date("j,n", $this->tempOffsetTimestamp));
                $this->moveableFeasts[] = array (
                    'id' => $values['id'], 
                    'day' =>$this->newDay,
                    'month' => $this->newMonth,
                    'title' => $values['title'],
                    'isRedLetter' => $values['isRedLetter'],
                    'isSubstitution' => $values['isSubstitution']
                );    
            } else {
                /*
                * All 'moveable' feasts are defined with two parameters. First is the 'bindingDate' string
                * which holds the following infomation: 'relativeDay', 'relativeMonth' and 'weekday'. The second is the
                * 'bindOffset' parameter which holds the value of how many 'weekdays' should the 'hitCounter' reach until
                * the algorythm declares a sucessful find while traveling into the past (backward search).
                * The algorithm will simply start from the given date defined with 'relativeDay' and 'relativeMonth' and
                * travel into the past, day by day, until it reaches the requested number of 'weekday' hits 
                * [Saturday, Sunday...]. If it does not find the date in the current year it will increment the year and
                * try again until it is successful.
                */
                $this->hitCount = 0; //will contain the number of 'weekday' hits
                $this->offsetCounter = 0; //counts the number of loop executions and holds the day decrementing value
                $this->offsetYear = $this->year; //we don't want to mess with the original value so we make a copy
                
                list($this->offsetDay, $this->offsetMonth, $this->weekday) = explode(',', $values['bindingDate']);
                
                do {
                    --$this->offsetCounter;
                    
                    $this->tempOffsetTimestamp = mktime(12, 0, 0, $this->offsetMonth, $this->offsetDay, $this->offsetYear) + $this->offsetCounter * 86400;
                    
                    /*
                    * The new set of date components representing the current day offset 
                    */
                    $this->tempOffsetDay = date("j", $this->tempOffsetTimestamp); 
                    $this->tempOffsetMonth = date("n", $this->tempOffsetTimestamp);
                    $this->tempOffsetYear = date("Y", $this->tempOffsetTimestamp);
                     
                    if($this->tempOffsetYear < $this->year){ //
                        /*
                        * If the feast is gone into the previous year we must use the next one as a base for 
                        * new calculations 
                        */
                        $this->offsetYear += 1;
                        
                        /*
                        * Since we are making a complete new set of calculations for the next year we
                        * need to reset these control variables
                        */
                        $this->offsetCounter = 0;
                        $this->hitCount = 0;
                        continue; //this will break the current 'do' loop and avoid changes to the 'hitCount'
                    }
                    
                    if($this->findWeekDay($this->tempOffsetDay, $this->tempOffsetMonth, $this->tempOffsetYear) == intval($this->weekday)) //is 'today' of type 'weekday'?
                        $this->hitCount++; //if yes, we found a 'weekday' so we increment the hitCount
                        
                } while ($this->hitCount < $values['bindOffset']);     
                /*
                * If everything went well during the trip into the past we are supposed to have the exact date we need 
                * to store in the 'moveable' feasts array.
                * NOTICE: Its perfectlly normal for some feasts not to appear in the current year so we need to be sure 
                * that only the ones in the current year are shown. That's the reason for the checkup on the next line.
                */
                
                if($this->tempOffsetYear == $this->year)
                    $this->moveableFeasts[] = array ( //off we go now (Nope... I don't like Warcraft :))!
                        'id' => $values['id'], 
                        'day' => $this->tempOffsetDay,
                        'month' => $this->tempOffsetMonth,
                        'title' => $values['title'],
                        'isRedLetter' => $values['isRedLetter'],
                        'isSubstitution' => $values['isSubstitution']
                    );
            }   
        }      
    }
    
    /**
     * Performs a replacement or modification of all the relevant 'fixed' feasts with their corresponding
     * 'moveable' versions. 
     * 
     * The resulting array will contain all the info required for the rendering functions of this class. 
     * NOTICE: Moveable days marked with 'isSubstitution' will completelly replace the data for the existing 'fixed' days
     * 'title' while those that are not will simply add the 'title' content to the existing 'fixed' day. If a 'moveable'
     * day is marked to just add it's 'title' content the array field will get an additional element called 'originalText'
     * which will store the text that is to be appended to the existing 'fixed' day title.
    */
    public function mergeFeasts(){
        foreach($this->loadedFixedFeasts as $feast => $values){
            $this->finalFeastsList[$feast] = $this->loadedFixedFeasts[$feast]; //copying the initial 'fixed' day value
            foreach($this->moveableFeasts as $movFeast => $movValues){
                if(($values['day'] == $movValues['day']) && ($values['month'] == $movValues['month'])){
                    if($movValues['isSubstitution']){ //full substitution?
                        $this->finalFeastsList[$feast] = $this->moveableFeasts[$movFeast];
                    } else {
                        $this->titleTemp = $values['title'];
                        $this->finalFeastsList[$feast] = $this->moveableFeasts[$movFeast];
                        $this->finalFeastsList[$feast]['originalTitle'] = $this->titleTemp;
                    }
                    break; //we don't want to waste cycles so we exit the loop after a successful find 
                }
            }   
        }        
    }
    
    /**
    * Renders the Orthodox church calendar.
    * 
    * Overloads the Calendar base class 'renderCalendar()' method. It makes use of the view templating system as a part
    * of a partial MVC design pattern. First, all of the required data is calculated and stored in a viewData variable
    * which is afterwards used to render the view template.
    * NOTICE: This method is required by the 'renderCalendarToXML()' method which uses it to create an XML response to the
    * AJAX requests.
    */
    public function renderCalendar(){        
        /**
         * Contains a complete list of attributes describing all of the month feasts
        */
        $this->viewData['finalFeastsList'] = array();
        
        /**
         * Contains the list of months along with an additional parameter that will be used to indicate the selected month
        */
        $this->viewData['monthSelect'] = array();   
        
        /*
         * First a view data variable is registered after a check is performed over an already existing viewData variable.
         * If such exists it is unset and afterwards a new one is registered.
        */
        if(isset($this->viewData)){
            unset($this->viewData);
        }
        
        /*
         * Including data into the array that has already been calculated
        */ 
        $this->viewData = array(
            'previousMonth' => $this->previousMonth,
            'previousYear' => $this->previousYear,
            'currentMonthName' => self::$months[$this->month - 1][0],
            'currentYear' => $this->year,
            'nextMonth' => $this->nextMonth,
            'nextYear' => $this->nextYear,
            'daysInMonth' => $this->daysInMonth,
            'abbrDays' => (($this->daysInMonth < 31) ? self::$abbrDays[1] : self::$abbrDays[0])    
        );
        
        /*
         * Calculating table display parameters for each feast and adding them to the 'viewData' array
        */
        foreach($this->finalFeastsList as $day => $values){
            /*
             * Is it sunday, a red letter or just a simple plain old day?
            */
            if(!$this->findWeekDay($values['day'], $this->month, $this->year)){
                $this->rowClass = "sunday";
                if($values['isRedLetter']){
                    $this->contentClass = "contentRedLetter";
                } else if($values['important']){
                    $this->contentClass = "contentImportant";   
                } else {
                    $this->contentClass = "contentSunday";   
                }
            } else if($values['isRedLetter']){
                $this->rowClass = "redLetter"; 
                if($values['important']){
                    $this->contentClass = "contentImportant";   
                } else {
                    $this->contentClass = "contentRedLetter";
                }
            } else {
                $this->rowClass = "normalDay";
                
                if($values['important']){
                    $this->contentClass = "contentImportant";
                } else {
                    $this->contentClass = "content";
                }
            }
                
           /*
            * Preparing the 'title' part which holds the actual feast title
           */
            if(isset($values['originalTitle'])){
                $this->titleTemp = $values['originalTitle']." - ".$values['title'];   
            } else {
                $this->titleTemp = $values['title'];   
            }
            
            /*
             * If a feast is described as important it will be emphasized
            */
            if($values['important'])
                $this->titleTemp = "<span class=\"important\">".htmlentities($this->titleTemp, ENT_QUOTES, APP_ENCODING)."</span>";
            
            $this->viewData['finalFeastsList'][] = array(
                'rowClass' => $this->rowClass,
                'weekdayNameShort' => self::$days[$this->findWeekDay($values['day'], $this->month, $this->year)][1],
                'day' => $values['day'],
                'month' => $values['month'],
                'year' => $this->year,
                'julianDay' => $this->calculateJulian($values['day']),
                'contentClass' => $this->contentClass,
                'titleTemp' => $this->titleTemp
            );                            
        }
        
        /*
         * Preparing data for the main calendar month select box and adding them to the 'viewData' array
        */
        foreach(self::$months as $key => $value){
            $this->viewData['monthSelect'][] = array(
                'value' => $key + 1,
                'name' => $value[0],
                'additional' => (($key + 1) == $this->month) ? ' selected="selected" ' : ''
            );
        }
            
        /*
        * Before we output the data throug the view we need to make sure that its clean of any possible malicius data. 
        */
        $this->cleanupOutput($this->viewData);
        
        /*
        * The following line does the actual view template loading. Mimics the MVC's view part.
        */
        $this->loadView('mainCalendar');
    }
    
    /**
     * Replaces the corresponding 'fixed' days with the 'moveable' alternative if one exists for the current day.
     * If a feast is marked as 'isSubstitution' then the 'fixed' feast will be completely replaced by the 'moveable'
     * alternative. If not, then the additional data from the 'moveable' tables will be appended to the 'fixed' days data.
    */
    protected function extractDetailsData($day){
        $this->mysqldb = new Database();
        
        foreach($this->loadedFixedFeasts as $feast => $values){
            if(($values['day'] == $day) && ($values['month'] == $this->month)){
                /*
                 * When a match is found we transfer the data to the '$this->details' and add the corresponding
                 * description field from the database.
                */
                $sql = "SELECT description FROM ".self::DB_TABLE_FIXED_FEASTS." WHERE id=".$values['id'];
                $this->result = $this->mysqldb->executeQuery($sql);
                
                $this->details['id'] = $values['id'];
                $this->details['day'] = $values['day'];
                $this->details['month'] = $values['month'];
                $this->details['fixedTitle'] = $values['title'];
                $this->details['fixedIsRedLetter'] = $values['isRedLetter'];
                $this->details['fixedImportant'] = $values['important'];
                
                $this->details['fixedDescription'] = $this->mysqldb->getFormattedResults(); //the result is multidimensional
                $this->details['fixedDescription'] = $this->details['fixedDescription'][0]['description']; //reassigning one dimensional value
                break; //breaking the loop when the result is found
            }    
        }
        
        /*
         * Now we search the 'moveable' feasts data for the date that might be added to the 'fixed' feast data. 
        */
        foreach($this->moveableFeasts as $feast => $values){
            if(($values['day'] == $day) && ($values['month'] == $this->month)){
                /*
                 * If a 'moveable' feast was found matching current 'fixed' feast's date we load its'title' 
                 * and 'description'. 
                */
                $sql = "SELECT description FROM ".self::DB_TABLE_MOVEABLE_FEASTS." WHERE id=".$values['id'];
                $this->result = $this->mysqldb->executeQuery($sql);
                
                $this->details['additionalId'] = $values['id'];
                $this->details['additionalTitle'] = $values['title'];
                $this->details['additionalIsRedLetter'] = $values['isRedLetter'];
                $this->details['additionalImportant'] = $values['important'];
                $this->details['isSubstitution'] = $values['isSubstitution'];
                
                $this->details['additionalDescription'] = $this->mysqldb->getFormattedResults(); 
                $this->details['additionalDescription'] = $this->details['additionalDescription'][0]['description'];
                break;
            }   
        }
        
        /*
         * Here we check some joining conditions. First we check if we need to add (or replace) the 'moveable' feast's 
         * title to (with) the 'fixed' one.
        */
        if(isset($this->details['additionalTitle'])){
            if($this->details['isSubstitution']){
                $this->details['title'] = $this->details['additionalTitle'];   
            } else {
                $this->details['title'] = $this->details['fixedTitle']." - ".$this->details['additionalTitle'];   
            }    
        } else {
            $this->details['title'] = $this->details['fixedTitle'];       
        }
        
        /*
         * Now we check the same condition for the description field.
        */
        if(isset($this->details['additionalDescription'])){
            if($this->details['isSubstitution']){
                $this->details['description'] = $this->details['additionalDescription'];   
            } else {
                $this->details['description'] = $this->details['fixedDescription']." - ".$this->details['additionalDescription'];   
            }    
        } else {
            $this->details['description'] = $this->details['fixedDescription'];   
        }
        
        /*
         * If at least one of the joining days is important the result must be important too. 
        */
        if(isset($this->details['additionalImportant'])){
            if($this->details['isSubstitution']){
                $this->details['important'] = $this->details['additionalImportant'];   
            } else {
                $this->details['important'] = $this->details['fixedImportant'] || $this->details['additionalImportant'];   
            }    
        } else {
            $this->details['important'] = $this->details['fixedImportant'];   
        }
        
        /*
         * If at least one of the joining days is a red letter the result must be a red letter too. 
        */
        if(isset($this->details['additionalIsRedLetter'])){
            if($this->details['isSubstitution']){
                $this->details['isRedLetter'] = $this->details['additionalIsRedLetter'];   
            } else {
                $this->details['isRedLetter'] = $this->details['fixedIsRedLetter'] || $this->details['additionalIsRedLetter'];   
            }    
        } else {
            $this->details['isRedLetter'] = $this->details['fixedIsRedLetter'];   
        }
        
        /*
         * If at least one of the joining letters is a red letter (or a substitution) then it's image is
         * going to be used as a feast image.
        */
        if(isset($this->details['additionalId'])){
            if($this->details['isSubstitution'] || $this->details['isRedLetter'] || $this->details['important']){
                $this->details['imageName'] = $this->details['additionalId'].".jpg";
            } else {
                $this->details['imageName'] = $this->details['id'].".jpg";
            }  
        } else {
            $this->details['imageName'] = $this->details['id'].".jpg";       
        }
            
        
        /*
         * If the title is too long we need to shorten it
        */
        if(mb_strlen($this->details['title'],"utf-8") > 60){
            $this->details['headerTitle'] = mb_substr($this->details['title'], 0, 57, "utf-8")."...";
        } else {
            $this->details['headerTitle'] = $this->details['title'];
        }    
    }
    
    /**
     * Prepares the data required by the details views.
     * 
     * This methods role is to gather all the required info for the view that is supposed to display it during the calendar
     * rendering. Take care that there are two methods that handle the rendering of the details, 
     * one used in case of a AJAX XML response and another one is used when there is no AJAX support (Javascript) available.
     * In first case both view templates must be loaded separately and in the later one the whole page is loaded.
     * That was the only way to evade creating separate methods just to render some HTML that would by the way again
     * be part of the PHP code. Not in this life buddy ;)
    */
    protected function prepareDetailsViewData($day){
        /*
         * Calculating some stuff required for the infoPanel navigation
        */
        list($this->prevDetailsDay, $this->prevDetailsMonth, $this->prevDetailsYear) = explode(",", date("j,n,Y", mktime(12,0,0,$this->details['month'],$day ,$this->year) - 86400));
        
        list($this->nextDetailsDay, $this->nextDetailsMonth, $this->nextDetailsYear) = explode(",", date("j,n,Y", mktime(12,0,0,$this->details['month'], $day, $this->year) + 86400));
        
        /*
         * First a view data variable is registered after a check is performed over an already existing viewData variable.
         * If such exists it is unset and afterwards a new one is registered.
        */
        if(isset($this->viewData)){
            unset($this->viewData);
        }
            
        /*
         * Populating the array with some nice view data
        */
        $this->viewData = array(
            'prevDetailsDay' => $this->prevDetailsDay,
            'prevDetailsMonth' => $this->prevDetailsMonth,
            'prevDetailsYear' => $this->prevDetailsYear,
            'nextDetailsDay' => $this->nextDetailsDay,
            'nextDetailsMonth' => $this->nextDetailsMonth,
            'nextDetailsYear' => $this->nextDetailsYear,
            'headerTitle' => $this->details['headerTitle'],
            'day' => $this->details['day'],
            'month' => $this->details['month'],
            'year' => $this->year,
            'isRedLetter' => (($this->details['isRedLetter']) ? ' class="redLetter"' : ''),
            'title' => $this->details['title'],
            'picture' => (file_exists(IMAGES_ROOT.$this->details['imageName']) ? IMAGES_URL.$this->details['imageName'] : 'images/nopic.jpg'),
            'picAlt' => (file_exists(IMAGES_ROOT.$this->details['imageName']) ? IMAGES_URL.$this->details['imageName'] : 'Нема слике'),
            'description' => $this->details['description']
        );            
    }
    
    /**
     * Renders a response XHTML for non AJAX details preview requests. 
    */
    public function renderDetails($day){
        /*
         * First we load the 'fixed' and 'moveable' data for the current date from the database
        */
        $this->loadDays();
        
        /*
         * Then we calculate the 'moveable' feasts for the current year 
        */
        $this->calculateMoveableFeasts();
        
        /*
         * Now we pick up the 'fixed' data for the current day and load the description part from the database
        */
        $this->extractDetailsData($day);
            
        /**
         *  Loading the number of records from both of the databases
        */
        $this->getNumberOfFeasts();
        
        /*
         * Preparing some stuff required for the infoPanel navigation
        */
        $this->prepareDetailsViewData($day);

        /*
        * Before we output the data throug the view we need to make sure that its clean of any possible malicius data. 
        */
        $this->cleanupOutput($this->viewData);
        
        /*
         * The actual output begins with loading the following view which renders a complete HTML page 
        * (hence this method is used for non AJAX requests).
        */
        $this->loadView('infoPanel');
    }
    
    /**
     * Renders a response XML for AJAX details preview requests. 
     * 
     * Creates the following structure:
     * <containers>
     *     <container>
     *          <title>The title of the div container which content is to be updated</title>
     *          <content>The updating content</content>
     *     </container>
     * </containers>
    */
    public function renderDetailsToXML($day){
        /*
         * First we load the 'fixed' and 'moveable' data for the current date from the database
        */
        $this->loadDays();
        
        /*
         * Then we calculate the 'moveable' feasts for the current year 
        */
        $this->calculateMoveableFeasts();
        
        /*
         * Now we pick up the 'fixed' data for the current day and load the description part from the database
        */
        $this->extractDetailsData($day);
            
        /*
         *  Loading the number of records from both of the databases
        */
        $this->getNumberOfFeasts();
        
        /*
         * Preparing some stuff required for the infoPanel navigation
        */
        $this->prepareDetailsViewData($day);

        /*
        * Before we output the data throug the view we need to make sure that its clean of any possible malicius data. 
        */
        $this->cleanupOutput($this->viewData);
        
        /*
         * First we send the required headers and the opening XML identification along with the encoding
        */
        header("Content-Type: text/xml"); //sending the content type header (required)
        header("Cache-Control: no-cache, must revalidate"); //preventing local caching
        header("Expires: Mon, 26 Jul 1997 05:00:00 GMT"); //making sure nothing is cached
        
        /*
         * The beggining of the output
        */
        echo '<?xml version="1.0" encoding="utf-8"?><containers>';
        
        /*
         *  The beggining of the 'infoPanelHeader' container
        */
        echo "<container><title>infoPanelHeader</title><content><![CDATA[";
        /*
         * The content of the infoPanelHeader is filled from the corresponding template file.  Here we load the
        * appropriate view.
        */
        $this->loadView('infoPanelHeader');
        echo "]]></content></container>";
        
        /*
         *  The beggining of the 'infoPanelContentContainer' container
        */
        echo "<container><title>infoPanelMainContent</title><content><![CDATA[";
        /*
         * The content of the infoPanelMainContent is filled from the corresponding template file. Here we load the
         * appropriate view. 
        */
        $this->loadView('infoPanelMainContent');
        echo "]]></content></container>"; 

        /*
         * The end of the output
        */
        echo '</containers>';                    
    }
    
    /**
     * Renders the calendar widget XHTML intended for the iframe site inclussion
    */
    public function renderWidget($day){
        /*
         * First we load the 'fixed' and 'moveable' data for the current date from the database
        */
        $this->loadDays();
        
        /*
         * Then we calculate the 'moveable' feasts for the current year 
        */
        $this->calculateMoveableFeasts();
        
        /*
         * Now we pick up the 'fixed' data for the current day and load the description part from the database
        */
        $this->extractDetailsData($day);
        
        /*
         * Preparing data for view output
        */
        if(isset($this->viewData)){
            unset($this->viewData);   
        }
        $this->viewData = array(
            'title' => $this->details['title'],
            'day' => $day,
            'month' => $this->month,
            'year' => $this->year,
            'picture' => (file_exists(WIDGET_IMAGES_ROOT.$this->details['imageName']) ? WIDGET_IMAGES_URL.$this->details['imageName'] : 'images/nopic_small.jpg'),
            'picAlt' => (file_exists(WIDGET_IMAGES_ROOT.$this->details['imageName']) ? WIDGET_IMAGES_URL.$this->details['imageName'] : 'Нема слике'),
            'classWarning' => $this->details['isRedLetter']  ? " class=\"warning\" " : '',
            'title' => $this->details['title'] 
        );
        
        /*
        * Before we output the data throug the view we need to make sure that its clean of any possible malicius data. 
        */
        $this->cleanupOutput($this->viewData);
        
        /*
         * Invoking the widget view loader
        */
        $this->loadView('widget');
    }
    
    /**
     * A view loader method.
     * 
     * This method is responsible for loading the specified template view and populating it with the supplied data
     * after which the HTML output is sent to the browser. This whole view loading story isnt't actually a must since
     * it would be perfectlly ok to simply require the external template file which already contains references to the
     * data that is supposed to be displayed in the template. Nevertheless this is a good way to show some good MVC habits :)
     * 
     * @param string $templateViewName The name of the view template to be loaded.
    */
    protected function loadView($templateViewName){
        try {
            if(file_exists(TEMPLATES_ROOT.$templateViewName.'.tmpl.php')){
                require_once(TEMPLATES_ROOT.$templateViewName.'.tmpl.php');       
            } else {
                die(TEMPLATES_ROOT.$templateViewName.'.tmpl.php');
                throw new Custom_Exception('view_template_does_not_exist');   
            }
        } catch (Custom_Exception $e){
            $e->noteMessage();
        }
    }
    
    /**
     * This method is used to clear the source variables recursivelly of any malicius input made by the user.
     * 
     * @param mixed $data A mixed variables source to be cleaned.
     * @return mixed The cleaned mixed variable.
    */
    protected function cleanupOutput($data){
        /*
         * For every element of the data that is supposed to be cleaned a check is performed to see if it is an array.
         * If the element is an array the cleanup method is called recursivelly on every element of it which also 
         * might be arrays. If the current element is not ana array then it means its elementary and the cleaning operation
         * can be performed on it.
         * The cleaning operation simply will try to perform an HTML named entities conversion (htmlentities) preventing
         * possible harmful XSS attacks.
        */
        foreach($data as $key => $value){
            /*
             * Is the current element an array?
            */
            if(is_array($value)){
                /*
                 * If yes then we call recursivelly this method and assign a resulting conversion array to the 
                 * current array element.
                */
                $data[$key] = $this->cleanupOutput($value);
            } else {
                /*
                 * If the element is not an array then we make the requested cleanup operation.
                */
                $data[$key] = htmlentities($value, ENT_QUOTES, APP_ENCODING);  
            }
        }           
        /*
         * The resulting data array is returned
        */
        return $data;
    }
}
?>
